# @small-tech/hyperscript-to-html-string

## Renders hyperscript to HTML string

This is a fork of [vhtml by Jason Miller](https://github.com/developit/vhtml) but, unlike vhtml, it does not carry out any sanitisation. Sanitisation should be carried out at a higher level – for example in your server-side framework. You should most likely be using Jason’s module unless you understand the differences and know why you might prefer this one.

This module exists mainly for use by [Kitten](https://codeberg.org/kitten/app)).

## What’s different

The differences are mostly due to Kitten’s use case of targetting HTML authoring on the server.

  - Does __not__ carry out sanitisation. Simply renders `hyperscript` (from e.g., htm or xhtm) into an HTML string. You must carry out sanisation of untrusted content before it reaches this module (e.g., in your authoring framework).

  - Boolean HTML attributes are output as just the name of the attribute, not `attribute="true"` (e.g.,`h('main', {hidden: true})` is rendered as `<div hidden></div>`, not `<div hidden="true"></div>`)
  - It does not include support for `dangerouslySetInnerHTML` (as everything is dangerously set because it does not carry out validation).
  
  - You can include `<style>` and `<script>` tags in your markup.

  - HTML entities are supported.

Other differences:

  - Moved repository to Codeberg.
  - Changed npm package name to @small-tech/hyperscript-to-html-string
  - Reset version to 1.0.0 for new fork.
  - Made the module ESM-only to keep things simple.
  - Source is now in the main directory, in a single file.
  - Tests are now in [tape](https://github.com/ljharb/tape) with [tap-monkey](https://github.com/small-tech/tap-monkey) reporter instead of mocha and chai.
  - Tests now use [xhtm](https://github.com/dy/xhtm) instead of JSX.
  - Tests are now in the main directory, in a single file.
  - Removed 18 development dependencies including babel, rollup, and uglify.

For full list of changes, please [see the changelog](https://codeberg.org/small-tech/vhtml/src/branch/master/CHANGELOG.md).

## Installation

Via npm:

`npm install @small-tech/vhtml`

## Usage

See [vhtml](https://github.com/developit/vhtml) but keep in mind that this module does not sanitise its content.

## License

MIT (as per the original module)
